/*
 *  DAApplicationMenu.m
 *  Spark Plugins
 *
 *  Created by Black Moon Team.
 *  Copyright (c) 2004 - 2007, Shadow Lab. All rights reserved.
 */

#import "DAApplicationMenu.h"

#import "DocumentAction.h"

#import <WonderBox/WonderBox.h>

@implementation DAApplicationMenu {
  BOOL _custom;
}

- (instancetype)initWithCoder:(NSCoder *)coder {
  if (self = [super initWithCoder:coder]) {
    [self removeAllItems];
    NSMenu *menu = [self menu];
    [self setAutoenablesItems:NO];
    [menu addItem:[NSMenuItem separatorItem]];
    /* Append choose menu item */
    NSString *title = NSLocalizedStringFromTableInBundle(@"CHOOSE_MENU", nil,
                                                         kDocumentActionBundle,
                                                         @"Title of the Choose MenuItem item in the (With Application:) Menu");
    NSMenuItem *item = [[NSMenuItem alloc] initWithTitle:title
                                                  action:@selector(choose:) 
                                           keyEquivalent:@""];
    item.target = self;
    [menu addItem:item];
    [self selectItemAtIndex:1];
    
    _custom = YES;
  }
  return self;
}

- (IBAction)choose:(id)sender {
  NSOpenPanel *oPanel = [NSOpenPanel openPanel];
  [oPanel setAllowsMultipleSelection:YES];
  oPanel.allowedFileTypes = @[@"app", @"APPL"];
  [oPanel beginSheetModalForWindow:self.window completionHandler:^(NSInteger result) {
    if (result == NSModalResponseCancel) {
      return;
    }
    if (!self->_custom) {
      [[self menu] insertItem:[NSMenuItem separatorItem] atIndex:0];
      self->_custom = YES;
    }
    NSURL *path;
    NSEnumerator *files = [[oPanel URLs] reverseObjectEnumerator];
    while (path = [files nextObject]) {
      [[self menu] insertItem:[self itemForURL:path] atIndex:0];
    }
    [self selectItemAtIndex:0];
  }];
}

- (void)loadAppForDocument:(NSURL *)url {
  while ([self numberOfItems] > 2) {
    [self removeItemAtIndex:0];
  }
  _custom = NO;
  if (url) {
    NSArray *applications = SPXCFArrayBridgingRelease(LSCopyApplicationURLsForURL(SPXNSToCFURL(url), kLSRolesAll));
    if ([applications count]) {
      /* Sort applications */
      NSSortDescriptor *desc = [[NSSortDescriptor alloc] initWithKey:@"path.lastPathComponent" ascending:NO selector:@selector(caseInsensitiveCompare:)];
      NSArray *sorted = [applications sortedArrayUsingDescriptors:@[desc]];
      NSEnumerator *urls = [sorted objectEnumerator];
      
      NSURL *application;
      while (application = [urls nextObject]) {
        if ([application isFileURL])
          [[self menu] insertItem:[self itemForURL:application] atIndex:0];
      }
      NSURL *prefered = SPXCFURLBridgingRelease(LSCopyDefaultApplicationURLForURL(SPXNSToCFURL(url), kLSRolesAll, NULL));
      if (prefered) {
        NSUInteger idx = [sorted indexOfObject:prefered];
        if (idx != NSNotFound) {
          idx++;
          [self selectItemAtIndex:[sorted count] - idx];
        }
      }
    }
  } else {
    [self selectItemAtIndex:1];
  }
}

- (NSMenuItem *)itemForURL:(NSURL *)path {
  NSString *name;
  if (![path getResourceValue:&name forKey:NSURLLocalizedNameKey error:NULL]) {
    name = [path lastPathComponent];
  }
  name = [name stringByDeletingPathExtension];
  
  NSImage *icon = [[NSWorkspace sharedWorkspace] iconForFile:[path path]];
  if (icon) {
    WBImageSetRepresentationsSize(icon, NSMakeSize(16, 16));
  }
  [icon setSize:NSMakeSize(16, 16)];
  
  NSMenuItem *item = [[NSMenuItem alloc] initWithTitle:name action:nil keyEquivalent:@""];
  item.representedObject = @{ @"path": path, @"name": name, @"icon": icon };
  item.image = icon;
  
  return item;
}

@end
